@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2014 John Darrington <jmd@gnu.org>
+;;; Copyright © 2015 Eric Bavier <bavier@member.fsf.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -18,6 +19,7 @@
 
 (define-module (gnu packages fltk)
   #:use-module (guix licenses)
+  #:use-module (gnu packages)
   #:use-module (gnu packages xorg)
   #:use-module (gnu packages gl)
   #:use-module (guix packages)
@@ -27,7 +29,7 @@
 (define-public fltk
   (package
     (name "fltk")
-    (version "1.3.2")
+    (version "1.3.3")
     (source
      (origin
       (method url-fetch)
@@ -35,14 +37,17 @@
                           "/fltk-" version "-source.tar.gz"))
       (sha256
        (base32
-        "1974brlk723095vf8z72kazq1cbqr9a51kq6b0xda6zkjkgl8q0p"))))
+        "15qd7lkz5d5ynz70xhxhigpz3wns39v9xcf7ggkl0792syc8sfgq"))
+      (patches (list (search-patch "fltk-shared-lib-defines.patch")))))
    (build-system gnu-build-system)
     (inputs
       `(("libx11" ,libx11)
         ("mesa" ,mesa)))
     (arguments
      `(#:tests? #f                      ;TODO: compile programs in "test" dir
-       #:configure-flags '("--enable-shared")
+       #:configure-flags
+       (list "--enable-shared"
+             (string-append "DSOFLAGS=-Wl,-rpath=" %output "/lib"))
        #:phases
        (alist-cons-before
         'configure 'patch-makeinclude
