@@ -1,7 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2014 Pjotr Prins <pjotr.guix@thebird.nl>
 ;;; Copyright © 2014 Ludovic Courtès <ludo@gnu.org>
-;;; Copyright © 2014 Mark H Weaver <mhw@netris.org>
+;;; Copyright © 2014, 2015 Mark H Weaver <mhw@netris.org>
 ;;; Copyright © 2014 David Thompson <davet@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -30,6 +30,7 @@
   #:use-module (gnu packages gdbm)
   #:use-module (guix packages)
   #:use-module (guix download)
+  #:use-module (guix git-download)
   #:use-module (guix utils)
   #:use-module (guix build-system gnu)
   #:use-module (guix build-system ruby))
@@ -37,7 +38,7 @@
 (define-public ruby
   (package
     (name "ruby")
-    (version "2.1.5")
+    (version "2.2.2")
     (source
      (origin
        (method url-fetch)
@@ -46,7 +47,7 @@
                            "/ruby-" version ".tar.xz"))
        (sha256
         (base32
-         "18z9mwhzgh4gqc0c6ydbjg0a87jc0j38sha10mzfvjbmsjw1xfi2"))))
+         "0qj48a8ji8qj1sllsrhb6y65frwr77bvr08xikj86w5mib8baczh"))))
     (build-system gnu-build-system)
     (arguments
      `(#:test-target "test"
@@ -114,6 +115,34 @@ a focus on simplicity and productivity.")
              (("/bin/sh") (which "sh"))))
          %standard-phases)))))
 
+(define-public ruby-hoe
+  (package
+    (name "ruby-hoe")
+    (version "3.13.1")
+    (source (origin
+              (method git-fetch)
+              (uri (git-reference
+                    (url "https://github.com/seattlerb/hoe.git")
+                    (commit "0c11836"))) ; no release tags :(
+              (sha256
+               (base32
+                "0i8dimf8kxcjgqj9x65bbi3l6hc9p9gbfbb1vmrz42764a4jjbz9"))) )
+    (build-system ruby-build-system)
+    (arguments
+     '(#:phases (alist-replace
+                 'build
+                 (lambda _ (zero? (system* "rake" "gem")))
+                 %standard-phases)))
+    (synopsis "Ruby project management helper")
+    (description
+     "Hoe is a rake/rubygems helper for project Rakefiles.  It helps manage,
+maintain, and release projects and includes a dynamic plug-in system allowing
+for easy extensibility.  Hoe ships with plug-ins for all the usual project
+tasks including rdoc generation, testing, packaging, deployment, and
+announcement.")
+    (home-page "http://www.zenspider.com/projects/hoe.html")
+    (license license:expat)))
+
 (define-public ruby-i18n
   (package
     (name "ruby-i18n")
@@ -122,6 +151,7 @@ a focus on simplicity and productivity.")
               (method url-fetch)
               (uri (string-append "https://github.com/svenfuchs/i18n/archive/v"
                                   version ".tar.gz"))
+              (file-name (string-append name "-" version ".tar.gz"))
               (sha256
                (base32
                 "1fdhnhh1p5g8vibv44d770z8nq208zrms3m2nswdvr54072y1m6k"))))
