@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013, 2014 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2014, 2015 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -20,6 +20,10 @@
   #:use-module (guix store)
   #:use-module (guix derivations)
   #:use-module (guix packages)
+  #:use-module (guix base32)
+  #:use-module (guix serialization)
+  #:use-module (guix hash)
+  #:use-module (guix build-system gnu)
   #:use-module (gnu packages bootstrap)
   #:use-module (srfi srfi-34)
   #:use-module (rnrs bytevectors)
@@ -27,9 +31,14 @@
   #:export (open-connection-for-tests
             random-text
             random-bytevector
+            network-reachable?
+            shebang-too-long?
             mock
+            %substitute-directory
             with-derivation-narinfo
-            dummy-package))
+            with-derivation-substitute
+            dummy-package
+            dummy-origin))
 
 ;;; Commentary:
 ;;;
@@ -71,6 +80,10 @@
             (loop (1+ i)))
           bv))))
 
+(define (network-reachable?)
+  "Return true if we can reach the Internet."
+  (false-if-exception (getaddrinfo "www.gnu.org" "80" AI_NUMERICSERV)))
+
 (define-syntax-rule (mock (module proc replacement) body ...)
   "Within BODY, replace the definition of PROC from MODULE with the definition
 given by REPLACEMENT."
@@ -86,28 +99,38 @@ given by REPLACEMENT."
 ;;; Narinfo files, as used by the substituter.
 ;;;
 
-(define* (derivation-narinfo drv #:optional (nar "example.nar"))
+(define* (derivation-narinfo drv #:key (nar "example.nar")
+                             (sha256 (make-bytevector 32 0)))
   "Return the contents of the narinfo corresponding to DRV; NAR should be the
-file name of the archive containing the substitute for DRV."
+file name of the archive containing the substitute for DRV, and SHA256 is the
+expected hash."
   (format #f "StorePath: ~a
 URL: ~a
 Compression: none
 NarSize: 1234
+NarHash: sha256:~a
 References: 
 System: ~a
 Deriver: ~a~%"
           (derivation->output-path drv)       ; StorePath
           nar                                 ; URL
+          (bytevector->nix-base32-string sha256)  ; NarHash
           (derivation-system drv)             ; System
           (basename
            (derivation-file-name drv))))      ; Deriver
 
-(define (call-with-derivation-narinfo drv thunk)
+(define %substitute-directory
+  (make-parameter
+   (and=> (getenv "GUIX_BINARY_SUBSTITUTE_URL")
+          (compose uri-path string->uri))))
+
+(define* (call-with-derivation-narinfo drv thunk
+                                       #:key (sha256 (make-bytevector 32 0)))
   "Call THUNK in a context where fake substituter data, as read by 'guix
-substitute-binary', has been installed for DRV."
+substitute', has been installed for DRV.  SHA256 is the hash of the
+expected output of DRV."
   (let* ((output  (derivation->output-path drv))
-         (dir     (uri-path
-                   (string->uri (getenv "GUIX_BINARY_SUBSTITUTE_URL"))))
+         (dir     (%substitute-directory))
          (info    (string-append dir "/nix-cache-info"))
          (narinfo (string-append dir "/" (store-path-hash-part output)
                                  ".narinfo")))
@@ -119,18 +142,74 @@ substitute-binary', has been installed for DRV."
                     (%store-prefix))))
         (call-with-output-file narinfo
           (lambda (p)
-            (display (derivation-narinfo drv) p))))
+            (display (derivation-narinfo drv #:sha256 sha256) p))))
       thunk
       (lambda ()
         (delete-file narinfo)
         (delete-file info)))))
 
-(define-syntax-rule (with-derivation-narinfo drv body ...)
-  "Evaluate BODY in a context where DRV looks substitutable from the
+(define-syntax with-derivation-narinfo
+  (syntax-rules (sha256 =>)
+    "Evaluate BODY in a context where DRV looks substitutable from the
 substituter's viewpoint."
-  (call-with-derivation-narinfo drv
+    ((_ drv (sha256 => hash) body ...)
+     (call-with-derivation-narinfo drv
+       (lambda () body ...)
+       #:sha256 hash))
+    ((_ drv body ...)
+     (call-with-derivation-narinfo drv
+       (lambda ()
+         body ...)))))
+
+(define* (call-with-derivation-substitute drv contents thunk
+                                          #:key sha256)
+  "Call THUNK in a context where a substitute for DRV has been installed,
+using CONTENTS, a string, as its contents.  If SHA256 is true, use it as the
+expected hash of the substitute; otherwise use the hash of the nar containing
+CONTENTS."
+  (define dir (%substitute-directory))
+  (dynamic-wind
     (lambda ()
-      body ...)))
+      (call-with-output-file (string-append dir "/example.out")
+        (lambda (port)
+          (display contents port)))
+      (call-with-output-file (string-append dir "/example.nar")
+        (lambda (p)
+          (write-file (string-append dir "/example.out") p))))
+    (lambda ()
+      (let ((hash (call-with-input-file (string-append dir "/example.nar")
+                    port-sha256)))
+        ;; Create fake substituter data, to be read by 'guix substitute'.
+        (call-with-derivation-narinfo drv
+          thunk
+          #:sha256 (or sha256 hash))))
+    (lambda ()
+      (delete-file (string-append dir "/example.out"))
+      (delete-file (string-append dir "/example.nar")))))
+
+(define (shebang-too-long?)
+  "Return true if the typical shebang in the current store would exceed
+Linux's static limit---the BINPRM_BUF_SIZE constant, normally 128 characters
+all included."
+  (define shebang
+    (string-append "#!" (%store-prefix) "/"
+                   (make-string 32 #\a)
+                   "-bootstrap-binaries-0/bin/bash\0"))
+
+  (> (string-length shebang) 128))
+
+(define-syntax with-derivation-substitute
+  (syntax-rules (sha256 =>)
+    "Evaluate BODY in a context where DRV is substitutable with the given
+CONTENTS."
+    ((_ drv contents (sha256 => hash) body ...)
+     (call-with-derivation-substitute drv contents
+       (lambda () body ...)
+       #:sha256 hash))
+    ((_ drv contents body ...)
+     (call-with-derivation-substitute drv contents
+       (lambda ()
+         body ...)))))
 
 (define-syntax-rule (dummy-package name* extra-fields ...)
   "Return a \"dummy\" package called NAME*, with all its compulsory fields
@@ -141,8 +220,16 @@ initialized with default values, and with EXTRA-FIELDS set as specified."
            (synopsis #f) (description #f)
            (home-page #f) (license #f)))
 
+(define-syntax-rule (dummy-origin extra-fields ...)
+  "Return a \"dummy\" origin, with all its compulsory fields initialized with
+default values, and with EXTRA-FIELDS set as specified."
+  (origin extra-fields ...
+          (method #f) (uri "http://www.example.com")
+          (sha256 (base32 (make-string 52 #\x)))))
+
 ;; Local Variables:
 ;; eval: (put 'call-with-derivation-narinfo 'scheme-indent-function 1)
+;; eval: (put 'call-with-derivation-substitute 'scheme-indent-function 2)
 ;; End:
 
 ;;; tests.scm ends here
