@@ -18,10 +18,7 @@ GCOptions::GCOptions()
 
 bool isInStore(const Path & path)
 {
-    return path[0] == '/'
-        && string(path, 0, settings.nixStore.size()) == settings.nixStore
-        && path.size() >= settings.nixStore.size() + 2
-        && path[settings.nixStore.size()] == '/';
+    return isInDir(path, settings.nixStore);
 }
 
 
@@ -319,15 +316,15 @@ void exportPaths(StoreAPI & store, const Paths & paths,
 namespace nix {
 
 
-boost::shared_ptr<StoreAPI> store;
+std::shared_ptr<StoreAPI> store;
 
 
-boost::shared_ptr<StoreAPI> openStore(bool reserveSpace)
+std::shared_ptr<StoreAPI> openStore(bool reserveSpace)
 {
     if (getEnv("NIX_REMOTE") == "")
-        return boost::shared_ptr<StoreAPI>(new LocalStore(reserveSpace));
+        return std::shared_ptr<StoreAPI>(new LocalStore(reserveSpace));
     else
-        return boost::shared_ptr<StoreAPI>(new RemoteStore());
+        return std::shared_ptr<StoreAPI>(new RemoteStore());
 }
 
 
