@@ -20,17 +20,10 @@
   #:use-module (guix import pypi)
   #:use-module (guix base32)
   #:use-module (guix hash)
+  #:use-module (guix tests)
   #:use-module (srfi srfi-64)
   #:use-module (ice-9 match))
 
-(define-syntax-rule (mock (module proc replacement) body ...)
-  (let* ((m (resolve-module 'module))
-         (original (module-ref m 'proc)))
-    (dynamic-wind
-      (lambda () (module-set! m 'proc replacement))
-      (lambda () body ...)
-      (lambda () (module-set! m 'proc original)))))
-
 (define test-json
   "{
   \"info\": {
@@ -60,7 +53,7 @@
 
 (test-assert "pypi->guix-package"
   ;; Replace network resources with sample data.
-  (mock ((guix import pypi) url-fetch
+  (mock ((guix import utils) url-fetch
          (lambda (url file-name)
            (with-output-to-file file-name
              (lambda ()
